package grammar;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

public abstract class Grammar {

	private String[] vars;
	private int alphaSize;
	private Map<String, List<List<String>>> rules;
	private String start;

	protected Grammar(String[] vars, int alphaSize, String start) {
		this.setVars(vars);
		this.setAlphaSize(alphaSize);
		this.setStart(start);
	}

	public String[] getVars() {
		return vars;
	}

	public void setVars(String[] vars) {
		this.vars = vars;
	}

	public int getAlphaSize() {
		return alphaSize;
	}

	public void setAlphaSize(int alphaSize) {
		this.alphaSize = alphaSize;
	}

	public Map<String, List<List<String>>> getRules() {
		return rules;
	}

	public void setRules(Map<String, List<List<String>>> rules) {
		this.rules = rules;
	}

	public String getStart() {
		return start;
	}

	public void setStart(String start) {
		this.start = start;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("G = (V, T, R, " + this.getStart() + ")\n");
		sb.append("V = {");
		for (int i = 0; i < this.getVars().length; i++) {
			sb.append(this.getVars()[i]);
			if (i < this.getVars().length - 1) {
				sb.append(", ");
			}
		}
		sb.append("}\n");
		sb.append("T = {");
		for (int i = 97; i < 97 + this.getAlphaSize(); i++) {
			sb.append((char) i);
			if (i < 96 + this.getAlphaSize()) {
				sb.append(", ");
			}
		}
		sb.append("}\n");
		sb.append("R = {");
		Iterator<Entry<String, List<List<String>>>> it = this.getRules().entrySet().iterator();
		while (it.hasNext()) {
			Entry<String, List<List<String>>> e = it.next();
			sb.append(e.getKey() + " -> ");
			List<List<String>> cs = e.getValue();
			for (int i = 0; i < cs.size(); i++) {
				List<String> c = cs.get(i);
				if (c.isEmpty()) {
					sb.append((char) 949);
				} else {
					for (int j = 0; j < c.size(); j++) {
						sb.append(c.get(j));
					}
				}

				if (i < cs.size() - 1) {
					sb.append(" | ");
				}
			}
			if (it.hasNext()) {
				sb.append(", ");
			}
		}
		sb.append("}");
		return sb.toString();
	}

}
